/**
 * ========================================
 * JFreeReport : a free Java report library
 * ========================================
 *
 * Project Info:  http://www.object-refinery.com/jfreereport/index.html
 * Project Lead:  Thomas Morgner (taquera@sherito.org);
 *
 * (C) Copyright 2000-2002, by Simba Management Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * ----------------
 * MfHeader.java
 * ----------------
 * (C)opyright 2002, by Thomas Morgner and Contributors.
 *
 * Original Author:  David R. Harris
 * Contributor(s):   Thomas Morgner
 *
 * $Id: MfHeader.java,v 1.2 2003/07/03 16:13:36 taqua Exp $
 *
 * Changes
 * -------
 */
package org.jfree.pixie.wmf;

import java.awt.Rectangle;
import java.io.IOException;
import java.io.InputStream;

/**
 A buffer which represents a Metafile header.
 */
public class MfHeader extends Buffer
{
  /** A constant stating that the given file is not Wmf-File at all.*/
  public static final int QUALITY_NO = 0;    // Can't convert.
  /** A constant stating that the given file could be a Wmf-File.*/
  public static final int QUALITY_MAYBE = 1; // Might be able to convert.
  /** A constant stating that the given file is a Wmf-File.*/
  public static final int QUALITY_YES = 2;   // Can convert.

  private static final int PLACEABLE_HEADER_SIZE = 22;
  private static final int STANDARD_HEADER_SIZE = 18;

  /** Metadata Positions ... */
  private static final int WMF_FILE_TYPE = 22;     // WORD
  private static final int WMF_HEADER_SIZE = 24;   // WORD
  private static final int WMF_VERSION = 26;       // WORD
  private static final int WMF_FILE_SIZE = 28;     // DWORD
  private static final int WMF_NUM_OF_REC = 32;    // WORD
  private static final int WMF_MAX_REC_SIZE = 34;  // DWORD
  private static final int WMF_NUM_PARAMS = 38;    // WORD always 0 not used

  /** MetaData type: WmfFile is a memory copy. */
  private static final int WMF_TYPE_MEM = 0;
  /** MetaData type: WmfFile is a disk copy. */
  private static final int WMF_TYPE_DISK = 1;

  /** A magic number indicating that this is a Aldus WMF file. */
  private static final int ALDUS_MAGIC_NUMBER_VAL = 0x9ac6cdd7;

  private static final int ALDUS_MAGIC_NUMBER_POS = 0;
  private static final int ALDUS_HANDLE_POS = 4;
  private static final int ALDUS_POS_LEFT = 6;
  private static final int ALDUS_POS_TOP = 8;
  private static final int ALDUS_POS_RIGHT = 10;
  private static final int ALDUS_POS_BOTTOM = 12;
  private static final int ALDUS_RESOLUTION = 14; // units per inch
  private static final int ALDUS_RESERVED = 16;
  private static final int ALDUS_CHECKSUM = 20;

  /**
   * Is the given input a metafile? We have to guess by reading the header
   * and/or by looking at the file name.
   *
   * @return either QUALITY_NO, QUALITY_MAYBE or QUALITY_YES.
   */
  public static int isMetafile (final String inName, final InputStream in)
  {
    if (in != null)
    {
      // See if we have a valid header.
      MfHeader header = new MfHeader ();
      try
      {
        in.mark (PLACEABLE_HEADER_SIZE + STANDARD_HEADER_SIZE);
        header.read (in);
      }
      catch (IOException e)
      {
        header = null;
      }
      finally
      {
        try
        {
          in.reset ();
        }
        catch (IOException e)
        {
        }
      }
      if (header == null || !header.isValid ())
        return QUALITY_NO;
      if (header.isPlaceable ())
        return QUALITY_YES;
      // We are not so confident of identifying non-placeable
      // metafiles, so we require both isValid() and the file
      // extension to match.
    }

    // True if the extension is .wmf.
    if (inName.regionMatches (true, inName.length () - 4, ".wmf", 0, 4))
    {
      return QUALITY_MAYBE;
    }

    return QUALITY_NO;
  }

  /**
   * Read the header from the given input.
   *
   * @param in the input stream
   * @throws IOException if an error occured.
   */
  public void read (final InputStream in) throws IOException
  {
    final int total = PLACEABLE_HEADER_SIZE + STANDARD_HEADER_SIZE;
    setCapacity (total);

    read (in, 0, 4);
    if (isPlaceable ())
    {
      read (in, 4, total - 4);
    }
    else
    {
      // Ignore the space for the placeable header
      for (int i = 0; i < 4; i++)
      {
        bytes[PLACEABLE_HEADER_SIZE + i] = bytes[i];
      }
      read (in, PLACEABLE_HEADER_SIZE + 4, STANDARD_HEADER_SIZE - 4);
    }

    // Now have the placeable header at the start of the headers buffer,
    // and the windows header following it.
  }

  /**
   * Return true if this is an Aldus placeable header.
   *
   * @return true, if this is an Aldus placeable header, false otherwise.
   */
  private boolean isPlaceable ()
  {
    // Verify magic number.
    return getInt (ALDUS_MAGIC_NUMBER_POS) == ALDUS_MAGIC_NUMBER_VAL;
  }

  /**
   * Returns true if it looks like a real metafile. This implementation does
   * not support Memory-WmfFiles.
   *
   * @return true, if this file is valid, false otherwise.
   */
  public boolean isValid ()
  {
    final int type = getShort (WMF_FILE_TYPE);  // Memory or disk.
    if (type == WMF_TYPE_MEM)
    {
      // type == null means this is a wmf from memory. we don't want that
      return false;
    }
    if (getShort (WMF_HEADER_SIZE) != 9)  // Header size.
    {
      // A VALID wmf-File has always a standard-header size of 9 WORDS == 18 bytes
      return false;
    }
    return true;
  }

  /**
   * Return the bounding box of this metafile.
   * This returns an empty (0,0,0,0) rectangle if this file is not placeable.
   *
   * @return the bounding box of the metafile.
   */
  public Rectangle getBBox ()
  {
    final int left = getShort (ALDUS_POS_LEFT);
    final int top = getShort (ALDUS_POS_TOP);
    final int right = getShort (ALDUS_POS_RIGHT);
    final int bottom = getShort (ALDUS_POS_BOTTOM);
    return new Rectangle (left, top, right - left, bottom - top);
  }

  /**
   * Gets the defined resolution, if this is an Aldus-File, null otherwise.
   *
   * @return the image resolution or 0 if not defined.
   */
  public int getUnitsPerInch ()
  {
    return getShort (ALDUS_RESOLUTION);
  }

  /**
   * Gets the file size of the WmfFile.
   *
   * @return the filesize in bytes.
   */
  public int getFileSize ()
  {
    return getInt (WMF_FILE_SIZE) * 2;
  }

  /**
   * Gets the number of records stored in this metafile.
   *
   * @return the number of records.
   */
  public int getObjectsSize ()
  {
    return getShort (WMF_NUM_OF_REC);
  }

  /**
   * Gets the size of the largest Record.
   *
   * @return the maximum record size.
   */
  public int getMaxRecordSize ()
  {
    return getInt (WMF_MAX_REC_SIZE) * 2;
  }

  /**
   * Gets the header size.
   *
   * @return the header size.
   */
  public int getHeaderSize ()
  {
    if (isPlaceable ())
      return PLACEABLE_HEADER_SIZE + STANDARD_HEADER_SIZE;
    else
      return STANDARD_HEADER_SIZE;
  }

}
